/*
* ===========================
* VDK Visual Development Kit
* Version 1.2.3
* October 1998, August 2000
* ===========================
*
* Copyright (C) 1998, Mario Motta
* Developed by Mario Motta <mmotta@guest.net>
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Library General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Library General Public License for more details.
*
* You should have received a copy of the GNU Library General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
* 02111-130
*/ 

#ifndef VDKSTRING_H
#define VDKSTRING_H

struct STRING 
{          
	char* s;              
	unsigned int ref ;    
};

/*!
\class VDKString
\brief Implements famous cont referenced string objects
*/
class VDKString 
{       
protected:
	STRING* p;         
public:
/*!
Constructor, makes an empty string
\code
VDKString s;
\endcode
*/
	VDKString();
/*!
Constructor
\param s a null terminated C string
\code
VDKString s = "uncle bill";
\endcode
*/

	VDKString (const char*s);
/*!
Constructor
\param c a single character
\code
VDKString s(c);
\endcode
*/

	VDKString (const char& c);
/*!
Copy-initializer
\param s a VDKString reference
\code
VDKString s = "uncle bill";
VDKString s1 = s;
\endcode
*/

	VDKString(const VDKString& s);

/*!
Assignement operator
\param s a VDKString reference
\code
VDKString s = "uncle bill";
VDKString s1 = s;
VDKString s2 = "uncle sam";
s = s2;
\endcode
*/
	VDKString& operator= (const VDKString& s);

/*!
Assignement operator
\param s a null terminated C string
\code
VDKString s = "uncle bill";
s = "uncle sam";
\endcode
*/
	VDKString& operator= (const char* s);

/*!
Destructor
*/
~VDKString();
/*!
VDKString to char* casting
** warning ** can violate data hiding OO concept
*/
	operator char*() { return p->s; }

/*!
Equality operator
*/
	int operator == (const VDKString& s) const ;
/*!
less than operator
*/
	int operator <( const VDKString& s) const ;
/*!
greater than operator
*/
	int operator>(const VDKString& s)  const ;
/*!
less-equal operator
*/
	int operator <=(const VDKString& s) const ;
/*!
greater-equal operator
*/
	int operator >=(const VDKString& s) const ;
/*!
disequality operator
*/
	int operator !=(const VDKString& s) const ;
/*!
cat to this
\param s a null terminated string
\code
VDKString s = "uncle bill";
s += " is a smart boy";
\endcode
*/
	VDKString& operator +=(const char* s);
/*!
cat to this
\param s a VDKString
\code
VDKString s = "uncle bill";
VDKString s1 = " is a smart boy";
s += s1;
\endcode
*/
	VDKString& operator +=(const VDKString& s);
/*!
Returns a VDKString concatenated\param s a null terminated string
\code
VDKString s = "uncle bill";
VDKString s1 = s + " is a smart boy";
\endcode
*/
	VDKString  operator + (const char* s) const;
	friend VDKString operator + (const char* s, const VDKString& vdks);
/*!
Returns a VDKString concatenated
\param s a VDKString
*/
	VDKString  operator +(const VDKString& s) const;
/*!
Returns true if this is an empty string
*/
	bool isNull() const;
/*!
as strlen()
*/
	int size() const;
/*!
index operator for const instances returns NULL if ix >= size
*/
char operator[](unsigned int ix) const;
/*! 
string pointer access for const instances
*/
const char* c_str() const;

};
#endif




